__author__ = 'heddevanderheide'

# Django specific
from django.conf import settings
from django.db import models

# App specific
from networth.managers import NetworthManager
from networth.mixins import NetworthMixin


class NetworthModel(NetworthMixin, models.Model):
    _networth = models.IntegerField(default=getattr(settings, 'NETWORTH_DEFAULT', 0))

    objects = NetworthManager()

    class Meta:
        abstract = True

    def relative_networth(self):
        # returns relative networth (percentual) compared to the
        # highest valued object

        ceiling = self.__class__.objects.ceiling()

        if ceiling == 0:
            return 100
        else:
            if self._networth:
                if self._networth == ceiling:
                    return 100

                return int((float(self._networth) / ceiling) * 100)

            return 0

    def _commit(self, n):
        self._networth = n
        self.save()


__author__ = 'heddevanderheide'

import unittest

# App specific
from networth.mixins import NetworthMixin


class TestObject(NetworthMixin):
    first_name = ''
    last_name = ''

    tags = None

    class Networth:
        fields = (
            ('first_name', (True, 1)),
            ('last_name', (lambda f: f.startswith('P'), 5)),
            ('tags', (lambda f: len(f), 'result')),
        )

    def __init__(self, **kwargs):
        self.first_name = kwargs.get('first_name', '')
        self.last_name = kwargs.get('last_name', '')
        self.tags = filter(None, kwargs.get('tags', '').split(','))

    def add_tag(self, tag):
        if self.tags:
            self.tags.append(tag)
        else:
            self.tags = [tag]


class TestNetworthMixin(unittest.TestCase):
    def setUp(self):
        self.obj_1 = TestObject(
            first_name='Pete'
        )
        self.obj_2 = TestObject(
            first_name='Pete',
            last_name='Philly'
        )
        self.obj_3 = TestObject(
            first_name='Pete',
            last_name='Philly',
            tags='foo'
        )

    def test_obj_1(self):
        self.assertEqual(self.obj_1.networth(), 1)

    def test_obj_2(self):
        self.assertEqual(self.obj_2.networth(), 6)

    def test_obj_3(self):
        self.assertEqual(self.obj_3.networth(), 7)

        self.obj_3.add_tag('bar')

        self.assertEqual(self.obj_3.networth(), 8)


__author__ = 'heddevanderheide'


class NetworthMixin(object):
    def networth(self, commit=False):
        return self.__networth(commit=commit)

    def __networth(self, commit=False):
        n = 0

        for field in self.__class__.Networth.fields:
            f, values = field

            v = getattr(self, f)

            definition, points = values
            definition_is_callable, points_is_callable = callable(definition), callable(points)

            if definition_is_callable:

                if values[1] == 'result':
                    n += definition(v)

                elif points_is_callable:
                    n += points(definition(v))

                else:
                    if v:
                        n += points

            else:

                if type(definition) == bool:
                    if points_is_callable:
                        n += points(1 if bool(v) == definition else 0)
                    else:
                        n += points

                elif points == 'result':
                    n += v

                elif points_is_callable:
                    n += points(v)

                else:
                    n += points

        if commit:
            self._commit(n)

        return n

    def _commit(self, n):
        raise NotImplementedError


from distutils.core import setup

setup(
    name='django-networth',
    version='0.5',
    packages=['networth'],
    url='https://github.com/Hedde/django-networth',
    license='MIT',
    author='Hedde van der Heide',
    author_email='me@heddevanderheide.nl',
    description='Valuate instances of Django Models.'
)


__author__ = 'heddevanderheide'

# Django specific
from django.db import models
from django.db.models import Max


class NetworthManager(models.Manager):
    def ceiling(self):
        return self.get_queryset().aggregate(Max('_networth'))['_networth__max']